/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.RichTextArea.Formatter;
import com.google.gwt.user.client.ui.ToggleButton;
import com.google.gwt.user.client.ui.Widget;

public class UiRichTextAreaToolbar extends Composite {

	interface DefaultUiBinder extends UiBinder<Widget, UiRichTextAreaToolbar> {
	}

	private static DefaultUiBinder uiBinder = GWT.create(DefaultUiBinder.class);

	private UiRichTextArea uiRichTextArea;
	
	private Formatter formatter;
	
	public UiRichTextAreaToolbar() {
		initWidget(uiBinder.createAndBindUi(this));
		showWYSIWYG.setVisible(false);
	}
	
	public void setUiRichTextArea(UiRichTextArea uiRichTextArea) {
		this.uiRichTextArea = uiRichTextArea;
		this.formatter = uiRichTextArea.getRichTextArea().getFormatter();
		
		this.uiRichTextArea.getRichTextArea().addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				updateStatus();
			}
		});
		
		this.uiRichTextArea.getRichTextArea().addKeyUpHandler(new KeyUpHandler() {
			public void onKeyUp(KeyUpEvent event) {
				updateStatus();
			}
		});
	}
	
	@UiField
	LLink showWYSIWYG;
	
	@UiHandler( "showWYSIWYG" )
	void showWYSIWYGClicked(ClickEvent event) {
		setWysiwygDisplayed(true);
	}	
	
	@UiField
	LLink showHTML;
	
	@UiHandler( "showHTML" )
	void showHTMLClicked(ClickEvent event) {
		setWysiwygDisplayed(false);
	}

	public void setWysiwygDisplayed(boolean wysiwygDisplayed) {
		uiRichTextArea.setWysiwygDisplayed(wysiwygDisplayed);
		showWYSIWYG.setVisible(!wysiwygDisplayed);
		showHTML.setVisible(wysiwygDisplayed);
	}
	
	@UiField
	ToggleButton bold;
	
	@UiHandler( "bold" )
	void boldClicked(ClickEvent event) {
		formatter.toggleBold();
	}	

	@UiField
	ToggleButton italic;
	
	@UiHandler( "italic" )
	void italicClicked(ClickEvent event) {
		formatter.toggleItalic();
	}	
	
	@UiField
	ToggleButton underline;
	
	@UiHandler( "underline" )
	void underlineClicked(ClickEvent event) {
		formatter.toggleUnderline();
	}	
	
	@UiField
	PushButton indent;
	
	@UiHandler( "indent" )
	void indentlineClicked(ClickEvent event) {
		formatter.rightIndent();
	}	
	
	@UiField
	PushButton outdent;
	
	@UiHandler( "outdent" )
	void outdentClicked(ClickEvent event) {
		formatter.leftIndent();
	}
	
	@UiField
	PushButton justifyLeft;

	@UiHandler( "justifyLeft" )
	void justifyLeftClicked(ClickEvent event) {
		formatter.setJustification(RichTextArea.Justification.LEFT);
	}

	@UiField
	PushButton justifyCenter;

	@UiHandler( "justifyCenter" )
	void justifyCenterClicked(ClickEvent event) {
		formatter.setJustification(RichTextArea.Justification.CENTER);
	}

	@UiField
	PushButton justifyRight;

	@UiHandler( "justifyRight" )
	void justifyRightClicked(ClickEvent event) {
		formatter.setJustification(RichTextArea.Justification.RIGHT);
	}

	@UiField
	PushButton hr;

	@UiHandler( "hr" )
	void hrClicked(ClickEvent event) {
		formatter.insertHorizontalRule();
	}

	@UiField
	PushButton ol;

	@UiHandler( "ol" )
	void olClicked(ClickEvent event) {
		formatter.insertOrderedList();
	}

	@UiField
	PushButton ul;

	@UiHandler( "ul" )
	void ulClicked(ClickEvent event) {
		formatter.insertUnorderedList();
	}

	@UiField
	PushButton insertImage;

	@UiHandler( "insertImage" )
	void insertImageClicked(ClickEvent event) {
		String url = Window.prompt("Enter an image URL:", "http://");
		if (url != null) {
			formatter.insertImage(url);
		}
	}

	@UiField
	PushButton createLink;

	@UiHandler( "createLink" )
	void createLinkClicked(ClickEvent event) {
		String url = Window.prompt("Enter a link URL:", "http://");
		if (url != null) {
			formatter.createLink(url);
		}
	}

	@UiField
	PushButton removeLink;

	@UiHandler( "removeLink" )
	void removeLinkClicked(ClickEvent event) {
		formatter.removeLink();
	}

	@UiField
	PushButton removeFormat;

	@UiHandler( "removeFormat" )
	void removeFormatClicked(ClickEvent event) {
		formatter.removeFormat();
	}

	private void updateStatus() {
		bold.setDown(formatter.isBold());
		italic.setDown(formatter.isItalic());
		underline.setDown(formatter.isUnderlined());
	}

}
